-- Camera Control
-- Designed by T.Iwata at DIGITALFILTER.COM 2012/06/27

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY CAM_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		PCLK	: IN std_logic;
		CamHsync	: IN std_logic;
		CamVsync	: IN std_logic;
		CamData	: IN std_logic_vector(7 downto 0);
		LB_WR_ADDR	: out std_logic_vector(9 downto 0);
		LB_WR_DATA	: out std_logic_vector(15 downto 0);
		LB_WR_N	: out std_logic;
		CamLineCount28 : out std_logic_vector(4 downto 0);
		CamHsync_EDGE: out std_logic;
		CamVsync_EDGE: out std_logic;
		CamPixEnb : out std_logic;
		CamPixCount : out std_logic_vector(10 downto 0); 
		CamLineCount : out std_logic_vector(8 downto 0)); 
END CAM_CTRL;

ARCHITECTURE structure OF CAM_CTRL IS

	signal PclkPixCount, PclkPixCount_oft, PclkPixCount_oft2 : std_logic_vector(11 downto 0);
	signal Rg_dec, gB_dec : std_logic;
	signal Rg_dec_dly1, Rg_dec_dly2 : std_logic;
	signal CamHsync_dly1, CamHsync_dly2 ,CamHsync_dly3 : std_logic;
	signal CamHsync_edge_sig : std_logic;
	signal Rg_latch, gB_latch : std_logic_vector(7 downto 0);
	signal CamVsync_dly1, CamVsync_dly2 : std_logic;
  signal CamVsync_edge_sig : std_logic;
	signal CamLineCount_sig : std_logic_vector(8 downto 0);
	signal address2write : std_logic_vector(9 downto 0);
	signal write_n : std_logic;
	signal CamLineCount28_sig : std_logic_vector(4 downto 0);

BEGIN
-------------------------------------------------------------- 
process( RST_N, PCLK, CamHsync_edge_sig )  
begin 
  if( RST_N = '0' or CamHsync_edge_sig = '1' ) then
    PclkPixCount <= (others => '0');
  elsif( PCLK'event and PCLK = '1' ) then
    PclkPixCount <= PclkPixCount + 1;
  end if;  
end process;

  PclkPixCount_oft <= PclkPixCount - 1920;
  PclkPixCount_oft2 <= PclkPixCount - 1919;
  address2write <= PclkPixCount_oft(10 downto 1);
  CamPixCount <= PclkPixCount_oft2(10 downto 0);
  CamPixEnb <= '1' when PclkPixCount >= 1919 and PclkPixCount < 1919+1280 else '0';

-------------------------------------------------------------- 
  Rg_dec <= PclkPixCount(0);
  gB_dec <= not PclkPixCount(0);
  
-------------------------------------------------------------- 
process( RST_N, PCLK )  
begin 
   if( RST_N = '0' ) then
     Rg_latch <= (others => '0');
   elsif( PCLK'event and PCLK = '1' ) then
    if(Rg_dec = '1') then
      Rg_latch <= CamData;
    end if;
   end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, PCLK )  
begin 
   if( RST_N = '0' ) then
     gB_latch <= (others => '0');
   elsif( PCLK'event and PCLK = '1' ) then
    if(gB_dec = '1') then
      gB_latch <= CamData;
    end if;
   end if;  
end process;
  
 
-------------------------------------------------------------- 
  LB_WR_N <= '0' when gB_dec = '0' and CamHsync_dly3 = '1' else '1';
 
--  LB_WR_DATA <= ("000000" & address2write);--Rg_latch & gB_latch;
  LB_WR_DATA <= Rg_latch & gB_latch;
  
  LB_WR_ADDR <= address2write;
	
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     CamHsync_dly3 <= '0';
     CamHsync_dly2 <= '0';
     CamHsync_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     CamHsync_dly3 <= CamHsync_dly2;
     CamHsync_dly2 <= CamHsync_dly1;
     CamHsync_dly1 <= CamHsync;
   end if;  
end process;

-------------------------------------------------------------- 
  CamHsync_edge_sig <= '1' when CamHsync_dly1 = '0' and CamHsync_dly2 = '1' else '0';
  
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     CamVsync_dly2 <= '0';
     CamVsync_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     CamVsync_dly2 <= CamVsync_dly1;
     CamVsync_dly1 <= CamVsync;
   end if;  
end process;

-------------------------------------------------------------- 
  CamVsync_edge_sig <= '1' when CamVsync_dly1 = '0' and CamVsync_dly2 = '1' else '0';
  CamVsync_EDGE <= CamVsync_edge_sig;
  
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
  if( RST_N = '0' ) then
    CamLineCount_sig <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(CamVsync_edge_sig = '1') then
      CamLineCount_sig <= (others => '0');
    elsif(CamHsync_edge_sig = '1') then
      CamLineCount_sig <= CamLineCount_sig + 1;
    end if;
  end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
  if( RST_N = '0' ) then
    CamLineCount28_sig <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(CamVsync_edge_sig = '1') then
      CamLineCount28_sig <= (others => '0');
    elsif(CamHsync_edge_sig = '1') then
      if(CamLineCount28_sig = 27) then
        CamLineCount28_sig <= (others => '0');
      else
        CamLineCount28_sig <= CamLineCount28_sig + 1;
      end if;
    end if;
  end if;  
end process;

  CamLineCount28 <= CamLineCount28_sig;

-------------------------------------------------------------- 
  CamLineCount <= CamLineCount_sig;
  CamHsync_EDGE <= CamHsync_edge_sig;
  
END structure;
