-- Display Control
-- Designed by T.Iwata at DIGITALFILTER.COM 2022/03/04

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY VGA_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
--		SoftReset_N	: IN std_logic;
--		BigLineN : out std_logic_vector(8 downto 0);
--		BigPixN : out std_logic_vector(9 downto 0);
--		DISPJUDGE	: out std_logic;
		CamHsync_EDGE	: IN std_logic;
		CamVsync_EDGE	: IN std_logic;
--		RD_ADDR_LOAD : out std_logic;
--		RD_ADDR_ENB : out std_logic;
--		WR_ADDR_CLR : out std_logic;
		VgaLineCount : out std_logic_vector(8 downto 0);
		VgaPixCount : out std_logic_vector(9 downto 0);
		VgaPixGate : out std_logic;
		VgaVisible : OUT std_logic;
		VgaVsync : OUT std_logic;
		VgaVsyncDisp : OUT std_logic;
		VgaHsync : OUT std_logic;
		VgaHsyncDisp : OUT std_logic;
		VgaVsync_edge : out std_logic;
		VgaHsync_edge : out std_logic );
	END VGA_CTRL;

ARCHITECTURE structure OF VGA_CTRL IS

	signal VgaLineCount_sig	: std_logic_vector(8 downto 0);
	signal VgaPixCount_sig	: std_logic_vector(9 downto 0);
	signal VgaPixCount_clr : std_logic;
	signal VgaLineCount_clr : std_logic;
	signal VgaPixCount_enb : std_logic;
	signal VgaLineCount_enb : std_logic;
	signal VgaVsync_sig, VgaVsyncDisp_sig : std_logic;
	signal VgaVsync_edge_sig : std_logic;
	signal VgaHsync_sig, VgaHsyncDisp_sig : std_logic;
	signal VgaHsync_edge_sig : std_logic;
	signal VgaFrameCount : std_logic_vector(1 downto 0);
	signal VgaFrameCount_enb : std_logic;
	signal VgaVisible_V : std_logic;
	signal VgaVisible_H : std_logic;
	signal count54_clr : std_logic;
	signal count54 : std_logic_vector(5 downto 0);
	signal count300 : std_logic_vector(8 downto 0);
	--signal BigPixN_sig : std_logic_vector(9 downto 0);
	--signal BigPixN_clr : std_logic;
	signal WR_N_X_sig : std_logic_vector(2 downto 0);

BEGIN
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
  if( RST_N = '0' ) then
    VgaPixCount_enb <= '0';
  elsif( CLK'event and CLK = '1' ) then
    VgaPixCount_enb <= not VgaPixCount_enb;
  end if; 
end process;

  VgaPixGate <= VgaPixCount_enb;

-------------------------------------------------------------- 
   VgaPixCount_clr <= '1' when (VgaPixCount_enb = '1' and VgaPixCount_sig = 799 ) else '0'; 
       -- from 783 2018/07/13
                   -- or CamHsync_EDGE = '1' else '0'; -- commented out 2018/07/13

--WR_ADDR_CLR <= BigPixN_clr;

--  RD_ADDR_LOAD <= '1' when BigLineN_L(1 downto 0) = "11" and VgaLineCount_enb = '1' else '0';

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then -- or SoftReset_N = '0') then
		VgaPixCount_sig <= (others => '1'); -- for soft-reset
	elsif( CLK'event and CLK = '1' ) then
		if(VgaPixCount_clr = '1') then 
			VgaPixCount_sig <= (others => '0');
		elsif(VgaPixCount_enb = '1') then 
			VgaPixCount_sig <= VgaPixCount_sig+ 1;
		end if; 
	end if;  
end process;

  VgaPixCount <= VgaPixCount_sig;

-------------------------------------------------------------- 
   VgaLineCount_enb <= '1' when VgaPixCount_sig = 799 and VgaPixCount_enb = '1' else '0'; -- from 783 2018/07/13
   VgaLineCount_clr <= '1' when VgaPixCount_clr = '1' and VgaLineCount_sig = 499 else '0'; 

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then -- or SoftReset_N = '0') then
		VgaLineCount_sig <= (others => '1'); -- for soft-reset
	elsif( CLK'event and CLK = '1' ) then
		if(VgaLineCount_clr = '1') then -- or CamVsync_edge = '1') then -- commented out 2018/07/13
			VgaLineCount_sig <= (others => '0');
		elsif(VgaLineCount_enb = '1') then 
			VgaLineCount_sig <= VgaLineCount_sig+ 1;
		end if; 
	end if;  
end process;

  VgaLineCount <= VgaLineCount_sig;
  
-------------------------------------------------------------- 
   VgaVisible_H <= '1' when VgaPixCount_sig >= 5 and VgaPixCount_sig < 645 else '0'; -- from 136 2012/09/28 
   VgaHsync_sig <= '1' when VgaPixCount_sig >= 95 else '0'; -- from 96 2019/12/30
   VgaHsyncDisp_sig <= '0' when VgaPixCount_sig >= 650 and VgaPixCount_sig < 650+95 else '1'; -- from 96 2019/12/30
   VgaHsync_edge_sig <= VgaPixCount_clr;
    -- '1' when VgaPixCount_sig = 3 and VgaPixCount_enb = '1' else '0'; -- 2020/01/03
   VgaHsync_edge <= VgaHsync_edge_sig;

-------------------------------------------------------------- 
   VgaVisible_V <= '1'when VgaLineCount_sig >= 0 and VgaLineCount_sig < 480 else '0'; -- 2020/01/05
--   VgaVisible_V <= '0'when VgaLineCount_sig >= 480 and VgaLineCount_sig < 500 else '1'; -- from 515 2012/09/28
   VgaVsync_sig <= '0' when VgaLineCount_sig >= 484 and VgaLineCount_sig <= 485 else '1'; -- 2012/11/07
   VgaVsyncDisp_sig <= '0' when VgaLineCount_sig >= 470 and VgaLineCount_sig <= 471 else '1'; -- 2020/01/05
   VgaVsync_edge_sig <= '1' when VgaLineCount_sig = 499 and VgaPixCount_sig = 799 and VgaPixCount_enb = '1' else '0'; -- 2022/01/05
   VgaVsync_edge <= VgaVsync_edge_sig;

-------------------------------------------------------------- 
gen_VgaVisible_sig : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VgaVisible <= '0';
	elsif( CLK'event and CLK = '1' ) then
	   if( VgaPixCount_enb = '1' ) then
		   VgaVisible <= VgaVisible_V and VgaVisible_H; 
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then -- or SoftReset_N = '0') then  -- or CamVsync_EDGE = '1') then --
		VgaFrameCount <= (others => '1'); -- for soft-reset
	elsif( CLK'event and CLK = '1' ) then
		if( VgaFrameCount_enb = '1' ) then
			VgaFrameCount <= VgaFrameCount + 1; 
		end if;
	end if;  
end process;

VgaFrameCount_enb <= '1' when VgaPixCount_enb = '1' and VgaLineCount_enb = '1' 
			and VgaLineCount_sig = 499 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VgaHsync <= '0';
		VgaHsyncDisp <= '0';
	elsif( CLK'event and CLK = '1' ) then
	   if( VgaPixCount_enb = '1' ) then
		   VgaHsync <= VgaHsync_sig; 
		   VgaHsyncDisp <= VgaHsyncDisp_sig; 
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VgaVsync <= '0';
		VgaVsyncDisp <= '0';
	elsif( CLK'event and CLK = '1' ) then
	   if( VgaPixCount_enb = '1' ) then
		   VgaVsync <= VgaVsync_sig; 
		   VgaVsyncDisp <= VgaVsyncDisp_sig; 
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
-- DISPJUDGE <= '1' when VgaFrameCount = "01" or VgaFrameCount = "11" else '0';

END structure;
