-- The layer5 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer5 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic_vector(2 downto 0);
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO128 : in std_logic_vector(6 downto 0);
  CO128_dly2 : in std_logic_vector(6 downto 0);
  X_RD_DATA : in std_logic_vector(639 downto 0); 
  ReluAll : out std_logic_vector(1279 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_layer5;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer5 is

component dotpro_W5
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO128is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end component;

component ROM_W5
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(767 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_00, mac_01, mac_02, mac_03, mac_04, mac_05, mac_06, mac_07, mac_08, mac_09 : std_logic_vector(13 downto 0);
signal mac_10, mac_11, mac_12, mac_13, mac_14, mac_15, mac_16, mac_17, mac_18, mac_19 : std_logic_vector(13 downto 0);
signal mac_20, mac_21, mac_22, mac_23, mac_24, mac_25, mac_26, mac_27, mac_28, mac_29 : std_logic_vector(13 downto 0);
signal mac_30, mac_31 : std_logic_vector(13 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09 : std_logic_vector(6 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15, truncate_16, truncate_17, truncate_18, truncate_19 : std_logic_vector(6 downto 0);
signal truncate_20, truncate_21, truncate_22, truncate_23, truncate_24, truncate_25, truncate_26, truncate_27, truncate_28, truncate_29 : std_logic_vector(6 downto 0);
signal truncate_30, truncate_31 : std_logic_vector(6 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09 : std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15, Relu_16, Relu_17, Relu_18, Relu_19 : std_logic_vector(4 downto 0);
signal Relu_20, Relu_21, Relu_22, Relu_23, Relu_24, Relu_25, Relu_26, Relu_27, Relu_28, Relu_29 : std_logic_vector(4 downto 0);
signal Relu_30, Relu_31 : std_logic_vector(4 downto 0);

signal outCh_000, outCh_001, outCh_002, outCh_003, outCh_004, outCh_005, outCh_006, outCh_007, outCh_008, outCh_009 : std_logic_vector(4 downto 0);

signal PixRomData : std_logic_vector(4 downto 0);
signal W5Addr, W5Addr_prev : std_logic_vector(10 downto 0);
signal W5Addr_L : std_logic_vector(3 downto 0);
signal W5Addr_H : std_logic_vector(10 downto 0);
signal W5Sel : std_logic_vector(2 downto 0);
signal OutChSel_dly1, OutChSel_dly2 : std_logic_vector(2 downto 0);
signal reg_00_0, reg_00_1, reg_00_2, reg_00_3, reg_00_4, reg_00_5, reg_00_6, reg_00_7 : std_logic_vector(4 downto 0);
signal reg_01_0, reg_01_1, reg_01_2, reg_01_3, reg_01_4, reg_01_5, reg_01_6, reg_01_7 : std_logic_vector(4 downto 0);
signal reg_02_0, reg_02_1, reg_02_2, reg_02_3, reg_02_4, reg_02_5, reg_02_6, reg_02_7 : std_logic_vector(4 downto 0);
signal reg_03_0, reg_03_1, reg_03_2, reg_03_3, reg_03_4, reg_03_5, reg_03_6, reg_03_7 : std_logic_vector(4 downto 0);
signal reg_04_0, reg_04_1, reg_04_2, reg_04_3, reg_04_4, reg_04_5, reg_04_6, reg_04_7 : std_logic_vector(4 downto 0);
signal reg_05_0, reg_05_1, reg_05_2, reg_05_3, reg_05_4, reg_05_5, reg_05_6, reg_05_7 : std_logic_vector(4 downto 0);
signal reg_06_0, reg_06_1, reg_06_2, reg_06_3, reg_06_4, reg_06_5, reg_06_6, reg_06_7 : std_logic_vector(4 downto 0);
signal reg_07_0, reg_07_1, reg_07_2, reg_07_3, reg_07_4, reg_07_5, reg_07_6, reg_07_7 : std_logic_vector(4 downto 0);
signal reg_08_0, reg_08_1, reg_08_2, reg_08_3, reg_08_4, reg_08_5, reg_08_6, reg_08_7 : std_logic_vector(4 downto 0);
signal reg_09_0, reg_09_1, reg_09_2, reg_09_3, reg_09_4, reg_09_5, reg_09_6, reg_09_7 : std_logic_vector(4 downto 0);
signal reg_10_0, reg_10_1, reg_10_2, reg_10_3, reg_10_4, reg_10_5, reg_10_6, reg_10_7 : std_logic_vector(4 downto 0);
signal reg_11_0, reg_11_1, reg_11_2, reg_11_3, reg_11_4, reg_11_5, reg_11_6, reg_11_7 : std_logic_vector(4 downto 0);
signal reg_12_0, reg_12_1, reg_12_2, reg_12_3, reg_12_4, reg_12_5, reg_12_6, reg_12_7 : std_logic_vector(4 downto 0);
signal reg_13_0, reg_13_1, reg_13_2, reg_13_3, reg_13_4, reg_13_5, reg_13_6, reg_13_7 : std_logic_vector(4 downto 0);
signal reg_14_0, reg_14_1, reg_14_2, reg_14_3, reg_14_4, reg_14_5, reg_14_6, reg_14_7 : std_logic_vector(4 downto 0);
signal reg_15_0, reg_15_1, reg_15_2, reg_15_3, reg_15_4, reg_15_5, reg_15_6, reg_15_7 : std_logic_vector(4 downto 0);
signal reg_16_0, reg_16_1, reg_16_2, reg_16_3, reg_16_4, reg_16_5, reg_16_6, reg_16_7 : std_logic_vector(4 downto 0);
signal reg_17_0, reg_17_1, reg_17_2, reg_17_3, reg_17_4, reg_17_5, reg_17_6, reg_17_7 : std_logic_vector(4 downto 0);
signal reg_18_0, reg_18_1, reg_18_2, reg_18_3, reg_18_4, reg_18_5, reg_18_6, reg_18_7 : std_logic_vector(4 downto 0);
signal reg_19_0, reg_19_1, reg_19_2, reg_19_3, reg_19_4, reg_19_5, reg_19_6, reg_19_7 : std_logic_vector(4 downto 0);
signal reg_20_0, reg_20_1, reg_20_2, reg_20_3, reg_20_4, reg_20_5, reg_20_6, reg_20_7 : std_logic_vector(4 downto 0);
signal reg_21_0, reg_21_1, reg_21_2, reg_21_3, reg_21_4, reg_21_5, reg_21_6, reg_21_7 : std_logic_vector(4 downto 0);
signal reg_22_0, reg_22_1, reg_22_2, reg_22_3, reg_22_4, reg_22_5, reg_22_6, reg_22_7 : std_logic_vector(4 downto 0);
signal reg_23_0, reg_23_1, reg_23_2, reg_23_3, reg_23_4, reg_23_5, reg_23_6, reg_23_7 : std_logic_vector(4 downto 0);
signal reg_24_0, reg_24_1, reg_24_2, reg_24_3, reg_24_4, reg_24_5, reg_24_6, reg_24_7 : std_logic_vector(4 downto 0);
signal reg_25_0, reg_25_1, reg_25_2, reg_25_3, reg_25_4, reg_25_5, reg_25_6, reg_25_7 : std_logic_vector(4 downto 0);
signal reg_26_0, reg_26_1, reg_26_2, reg_26_3, reg_26_4, reg_26_5, reg_26_6, reg_26_7 : std_logic_vector(4 downto 0);
signal reg_27_0, reg_27_1, reg_27_2, reg_27_3, reg_27_4, reg_27_5, reg_27_6, reg_27_7 : std_logic_vector(4 downto 0);
signal reg_28_0, reg_28_1, reg_28_2, reg_28_3, reg_28_4, reg_28_5, reg_28_6, reg_28_7 : std_logic_vector(4 downto 0);
signal reg_29_0, reg_29_1, reg_29_2, reg_29_3, reg_29_4, reg_29_5, reg_29_6, reg_29_7 : std_logic_vector(4 downto 0);
signal reg_30_0, reg_30_1, reg_30_2, reg_30_3, reg_30_4, reg_30_5, reg_30_6, reg_30_7 : std_logic_vector(4 downto 0);
signal reg_31_0, reg_31_1, reg_31_2, reg_31_3, reg_31_4, reg_31_5, reg_31_6, reg_31_7 : std_logic_vector(4 downto 0);
signal enb_a, enb_b, enb_c, enb_d : std_logic;
signal enb_e, enb_f, enb_g, enb_h : std_logic;
signal CO9is1, CO128is0 : std_logic;
signal W5Data : std_logic_vector(767 downto 0);
signal coeff_00, coeff_01, coeff_02, coeff_03, coeff_04, coeff_05, coeff_06, coeff_07, coeff_08, coeff_09: std_logic_vector(2 downto 0);
signal coeff_10, coeff_11, coeff_12, coeff_13, coeff_14, coeff_15, coeff_16, coeff_17, coeff_18, coeff_19: std_logic_vector(2 downto 0);
signal coeff_20, coeff_21, coeff_22, coeff_23, coeff_24, coeff_25, coeff_26, coeff_27, coeff_28, coeff_29: std_logic_vector(2 downto 0);
signal coeff_30, coeff_31, coeff_32, coeff_33, coeff_34, coeff_35, coeff_36, coeff_37, coeff_38, coeff_39: std_logic_vector(2 downto 0);
signal coeff_40, coeff_41, coeff_42, coeff_43, coeff_44, coeff_45, coeff_46, coeff_47, coeff_48, coeff_49: std_logic_vector(2 downto 0);
signal coeff_50, coeff_51, coeff_52, coeff_53, coeff_54, coeff_55, coeff_56, coeff_57, coeff_58, coeff_59: std_logic_vector(2 downto 0);
signal coeff_60, coeff_61, coeff_62, coeff_63: std_logic_vector(2 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
CO128is0 <= '1' when CO128 = 0 else '0';

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= (others => '0');
		OutChSel_dly1 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

W5Sel <= OutChSel_dly2;

---------------------------------------------------------
  PixRomData <= 
		X_RD_DATA(	639	downto	635	) when CO128_dly2 = "0000000" else 
		X_RD_DATA(	634	downto	630	) when CO128_dly2 = "0000001" else 
		X_RD_DATA(	629	downto	625	) when CO128_dly2 = "0000010" else 
		X_RD_DATA(	624	downto	620	) when CO128_dly2 = "0000011" else 
		X_RD_DATA(	619	downto	615	) when CO128_dly2 = "0000100" else 
		X_RD_DATA(	614	downto	610	) when CO128_dly2 = "0000101" else 
		X_RD_DATA(	609	downto	605	) when CO128_dly2 = "0000110" else 
		X_RD_DATA(	604	downto	600	) when CO128_dly2 = "0000111" else 
		X_RD_DATA(	599	downto	595	) when CO128_dly2 = "0001000" else 
		X_RD_DATA(	594	downto	590	) when CO128_dly2 = "0001001" else 
		X_RD_DATA(	589	downto	585	) when CO128_dly2 = "0001010" else 
		X_RD_DATA(	584	downto	580	) when CO128_dly2 = "0001011" else 
		X_RD_DATA(	579	downto	575	) when CO128_dly2 = "0001100" else 
		X_RD_DATA(	574	downto	570	) when CO128_dly2 = "0001101" else 
		X_RD_DATA(	569	downto	565	) when CO128_dly2 = "0001110" else 
		X_RD_DATA(	564	downto	560	) when CO128_dly2 = "0001111" else 
		X_RD_DATA(	559	downto	555	) when CO128_dly2 = "0010000" else 
		X_RD_DATA(	554	downto	550	) when CO128_dly2 = "0010001" else 
		X_RD_DATA(	549	downto	545	) when CO128_dly2 = "0010010" else 
		X_RD_DATA(	544	downto	540	) when CO128_dly2 = "0010011" else 
		X_RD_DATA(	539	downto	535	) when CO128_dly2 = "0010100" else 
		X_RD_DATA(	534	downto	530	) when CO128_dly2 = "0010101" else 
		X_RD_DATA(	529	downto	525	) when CO128_dly2 = "0010110" else 
		X_RD_DATA(	524	downto	520	) when CO128_dly2 = "0010111" else 
		X_RD_DATA(	519	downto	515	) when CO128_dly2 = "0011000" else 
		X_RD_DATA(	514	downto	510	) when CO128_dly2 = "0011001" else 
		X_RD_DATA(	509	downto	505	) when CO128_dly2 = "0011010" else 
		X_RD_DATA(	504	downto	500	) when CO128_dly2 = "0011011" else 
		X_RD_DATA(	499	downto	495	) when CO128_dly2 = "0011100" else 
		X_RD_DATA(	494	downto	490	) when CO128_dly2 = "0011101" else 
		X_RD_DATA(	489	downto	485	) when CO128_dly2 = "0011110" else 
		X_RD_DATA(	484	downto	480	) when CO128_dly2 = "0011111" else 
		X_RD_DATA(	479	downto	475	) when CO128_dly2 = "0100000" else 
		X_RD_DATA(	474	downto	470	) when CO128_dly2 = "0100001" else 
		X_RD_DATA(	469	downto	465	) when CO128_dly2 = "0100010" else 
		X_RD_DATA(	464	downto	460	) when CO128_dly2 = "0100011" else 
		X_RD_DATA(	459	downto	455	) when CO128_dly2 = "0100100" else 
		X_RD_DATA(	454	downto	450	) when CO128_dly2 = "0100101" else 
		X_RD_DATA(	449	downto	445	) when CO128_dly2 = "0100110" else 
		X_RD_DATA(	444	downto	440	) when CO128_dly2 = "0100111" else 
		X_RD_DATA(	439	downto	435	) when CO128_dly2 = "0101000" else 
		X_RD_DATA(	434	downto	430	) when CO128_dly2 = "0101001" else 
		X_RD_DATA(	429	downto	425	) when CO128_dly2 = "0101010" else 
		X_RD_DATA(	424	downto	420	) when CO128_dly2 = "0101011" else 
		X_RD_DATA(	419	downto	415	) when CO128_dly2 = "0101100" else 
		X_RD_DATA(	414	downto	410	) when CO128_dly2 = "0101101" else 
		X_RD_DATA(	409	downto	405	) when CO128_dly2 = "0101110" else 
		X_RD_DATA(	404	downto	400	) when CO128_dly2 = "0101111" else 
		X_RD_DATA(	399	downto	395	) when CO128_dly2 = "0110000" else 
		X_RD_DATA(	394	downto	390	) when CO128_dly2 = "0110001" else 
		X_RD_DATA(	389	downto	385	) when CO128_dly2 = "0110010" else 
		X_RD_DATA(	384	downto	380	) when CO128_dly2 = "0110011" else 
		X_RD_DATA(	379	downto	375	) when CO128_dly2 = "0110100" else 
		X_RD_DATA(	374	downto	370	) when CO128_dly2 = "0110101" else 
		X_RD_DATA(	369	downto	365	) when CO128_dly2 = "0110110" else 
		X_RD_DATA(	364	downto	360	) when CO128_dly2 = "0110111" else 
		X_RD_DATA(	359	downto	355	) when CO128_dly2 = "0111000" else 
		X_RD_DATA(	354	downto	350	) when CO128_dly2 = "0111001" else 
		X_RD_DATA(	349	downto	345	) when CO128_dly2 = "0111010" else 
		X_RD_DATA(	344	downto	340	) when CO128_dly2 = "0111011" else 
		X_RD_DATA(	339	downto	335	) when CO128_dly2 = "0111100" else 
		X_RD_DATA(	334	downto	330	) when CO128_dly2 = "0111101" else 
		X_RD_DATA(	329	downto	325	) when CO128_dly2 = "0111110" else 
		X_RD_DATA(	324	downto	320	) when CO128_dly2 = "0111111" else 
		X_RD_DATA(	319	downto	315	) when CO128_dly2 = "1000000" else 
		X_RD_DATA(	314	downto	310	) when CO128_dly2 = "1000001" else 
		X_RD_DATA(	309	downto	305	) when CO128_dly2 = "1000010" else 
		X_RD_DATA(	304	downto	300	) when CO128_dly2 = "1000011" else 
		X_RD_DATA(	299	downto	295	) when CO128_dly2 = "1000100" else 
		X_RD_DATA(	294	downto	290	) when CO128_dly2 = "1000101" else 
		X_RD_DATA(	289	downto	285	) when CO128_dly2 = "1000110" else 
		X_RD_DATA(	284	downto	280	) when CO128_dly2 = "1000111" else 
		X_RD_DATA(	279	downto	275	) when CO128_dly2 = "1001000" else 
		X_RD_DATA(	274	downto	270	) when CO128_dly2 = "1001001" else 
		X_RD_DATA(	269	downto	265	) when CO128_dly2 = "1001010" else 
		X_RD_DATA(	264	downto	260	) when CO128_dly2 = "1001011" else 
		X_RD_DATA(	259	downto	255	) when CO128_dly2 = "1001100" else 
		X_RD_DATA(	254	downto	250	) when CO128_dly2 = "1001101" else 
		X_RD_DATA(	249	downto	245	) when CO128_dly2 = "1001110" else 
		X_RD_DATA(	244	downto	240	) when CO128_dly2 = "1001111" else 
		X_RD_DATA(	239	downto	235	) when CO128_dly2 = "1010000" else 
		X_RD_DATA(	234	downto	230	) when CO128_dly2 = "1010001" else 
		X_RD_DATA(	229	downto	225	) when CO128_dly2 = "1010010" else 
		X_RD_DATA(	224	downto	220	) when CO128_dly2 = "1010011" else 
		X_RD_DATA(	219	downto	215	) when CO128_dly2 = "1010100" else 
		X_RD_DATA(	214	downto	210	) when CO128_dly2 = "1010101" else 
		X_RD_DATA(	209	downto	205	) when CO128_dly2 = "1010110" else 
		X_RD_DATA(	204	downto	200	) when CO128_dly2 = "1010111" else 
		X_RD_DATA(	199	downto	195	) when CO128_dly2 = "1011000" else 
		X_RD_DATA(	194	downto	190	) when CO128_dly2 = "1011001" else 
		X_RD_DATA(	189	downto	185	) when CO128_dly2 = "1011010" else 
		X_RD_DATA(	184	downto	180	) when CO128_dly2 = "1011011" else 
		X_RD_DATA(	179	downto	175	) when CO128_dly2 = "1011100" else 
		X_RD_DATA(	174	downto	170	) when CO128_dly2 = "1011101" else 
		X_RD_DATA(	169	downto	165	) when CO128_dly2 = "1011110" else 
		X_RD_DATA(	164	downto	160	) when CO128_dly2 = "1011111" else 
		X_RD_DATA(	159	downto	155	) when CO128_dly2 = "1100000" else 
		X_RD_DATA(	154	downto	150	) when CO128_dly2 = "1100001" else 
		X_RD_DATA(	149	downto	145	) when CO128_dly2 = "1100010" else 
		X_RD_DATA(	144	downto	140	) when CO128_dly2 = "1100011" else 
		X_RD_DATA(	139	downto	135	) when CO128_dly2 = "1100100" else 
		X_RD_DATA(	134	downto	130	) when CO128_dly2 = "1100101" else 
		X_RD_DATA(	129	downto	125	) when CO128_dly2 = "1100110" else 
		X_RD_DATA(	124	downto	120	) when CO128_dly2 = "1100111" else 
		X_RD_DATA(	119	downto	115	) when CO128_dly2 = "1101000" else 
		X_RD_DATA(	114	downto	110	) when CO128_dly2 = "1101001" else 
		X_RD_DATA(	109	downto	105	) when CO128_dly2 = "1101010" else 
		X_RD_DATA(	104	downto	100	) when CO128_dly2 = "1101011" else 
		X_RD_DATA(	99	downto	95	) when CO128_dly2 = "1101100" else 
		X_RD_DATA(	94	downto	90	) when CO128_dly2 = "1101101" else 
		X_RD_DATA(	89	downto	85	) when CO128_dly2 = "1101110" else 
		X_RD_DATA(	84	downto	80	) when CO128_dly2 = "1101111" else 
		X_RD_DATA(	79	downto	75	) when CO128_dly2 = "1110000" else 
		X_RD_DATA(	74	downto	70	) when CO128_dly2 = "1110001" else 
		X_RD_DATA(	69	downto	65	) when CO128_dly2 = "1110010" else 
		X_RD_DATA(	64	downto	60	) when CO128_dly2 = "1110011" else 
		X_RD_DATA(	59	downto	55	) when CO128_dly2 = "1110100" else 
		X_RD_DATA(	54	downto	50	) when CO128_dly2 = "1110101" else 
		X_RD_DATA(	49	downto	45	) when CO128_dly2 = "1110110" else 
		X_RD_DATA(	44	downto	40	) when CO128_dly2 = "1110111" else 
		X_RD_DATA(	39	downto	35	) when CO128_dly2 = "1111000" else 
		X_RD_DATA(	34	downto	30	) when CO128_dly2 = "1111001" else 
		X_RD_DATA(	29	downto	25	) when CO128_dly2 = "1111010" else 
		X_RD_DATA(	24	downto	20	) when CO128_dly2 = "1111011" else 
		X_RD_DATA(	19	downto	15	) when CO128_dly2 = "1111100" else 
		X_RD_DATA(	14	downto	10	) when CO128_dly2 = "1111101" else 
		X_RD_DATA(	9	downto	5	) when CO128_dly2 = "1111110" else 
		X_RD_DATA(	4	downto	0	); 

W5Addr_L <= CO9;
W5Addr_H <= ("0" & CO128 & "000") + ("0000" & CO128); -- co128 x 9
W5Addr_prev <= W5Addr_H + ("000000" & W5Addr_L);

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		W5Addr <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(W5Addr_prev >= -448 and W5Addr_prev <= -1) then -- note that this is Signed
			W5Addr <= (others => '0');
		else
			W5Addr <= W5Addr_prev;
		end if;
	end if;  
end process;

---------------------------------------------------------
enb_a <= '1' when W5Sel = 1 and CO128 = 127 and CO9 = 8 else '0';
enb_b <= '1' when W5Sel = 2 and CO128 = 127 and CO9 = 8 else '0';
enb_c <= '1' when W5Sel = 3 and CO128 = 127 and CO9 = 8 else '0';
enb_d <= '1' when W5Sel = 4 and CO128 = 127 and CO9 = 8 else '0';
enb_e <= '1' when W5Sel = 5 and CO128 = 127 and CO9 = 8 else '0';
enb_f <= '1' when W5Sel = 6 and CO128 = 127 and CO9 = 8 else '0';
enb_g <= '1' when W5Sel = 7 and CO128 = 127 and CO9 = 8 else '0';
enb_h <= '1' when W5Sel = 0 and CO128 = 127 and CO9 = 8 else '0';

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			outCh_000 <= reg_00_0;
			outCh_001 <= reg_00_1;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_00_0 <= Relu_00;
		elsif(enb_b = '1') then
			reg_00_1 <= Relu_00;
		elsif(enb_c = '1') then
			reg_00_2 <= Relu_00;
		elsif(enb_d = '1') then
			reg_00_3 <= Relu_00;
		elsif(enb_e = '1') then
			reg_00_4 <= Relu_00;
		elsif(enb_f = '1') then
			reg_00_5 <= Relu_00;
		elsif(enb_g = '1') then
			reg_00_6 <= Relu_00;
		elsif(enb_h = '1') then
			reg_00_7 <= Relu_00;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_01_0 <= Relu_01;
		elsif(enb_b = '1') then
			reg_01_1 <= Relu_01;
		elsif(enb_c = '1') then
			reg_01_2 <= Relu_01;
		elsif(enb_d = '1') then
			reg_01_3 <= Relu_01;
		elsif(enb_e = '1') then
			reg_01_4 <= Relu_01;
		elsif(enb_f = '1') then
			reg_01_5 <= Relu_01;
		elsif(enb_g = '1') then
			reg_01_6 <= Relu_01;
		elsif(enb_h = '1') then
			reg_01_7 <= Relu_01;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_02_0 <= Relu_02;
		elsif(enb_b = '1') then
			reg_02_1 <= Relu_02;
		elsif(enb_c = '1') then
			reg_02_2 <= Relu_02;
		elsif(enb_d = '1') then
			reg_02_3 <= Relu_02;
		elsif(enb_e = '1') then
			reg_02_4 <= Relu_02;
		elsif(enb_f = '1') then
			reg_02_5 <= Relu_02;
		elsif(enb_g = '1') then
			reg_02_6 <= Relu_02;
		elsif(enb_h = '1') then
			reg_02_7 <= Relu_02;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_03_0 <= Relu_03;
		elsif(enb_b = '1') then
			reg_03_1 <= Relu_03;
		elsif(enb_c = '1') then
			reg_03_2 <= Relu_03;
		elsif(enb_d = '1') then
			reg_03_3 <= Relu_03;
		elsif(enb_e = '1') then
			reg_03_4 <= Relu_03;
		elsif(enb_f = '1') then
			reg_03_5 <= Relu_03;
		elsif(enb_g = '1') then
			reg_03_6 <= Relu_03;
		elsif(enb_h = '1') then
			reg_03_7 <= Relu_03;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_04_0 <= Relu_04;
		elsif(enb_b = '1') then
			reg_04_1 <= Relu_04;
		elsif(enb_c = '1') then
			reg_04_2 <= Relu_04;
		elsif(enb_d = '1') then
			reg_04_3 <= Relu_04;
		elsif(enb_e = '1') then
			reg_04_4 <= Relu_04;
		elsif(enb_f = '1') then
			reg_04_5 <= Relu_04;
		elsif(enb_g = '1') then
			reg_04_6 <= Relu_04;
		elsif(enb_h = '1') then
			reg_04_7 <= Relu_04;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_05_0 <= Relu_05;
		elsif(enb_b = '1') then
			reg_05_1 <= Relu_05;
		elsif(enb_c = '1') then
			reg_05_2 <= Relu_05;
		elsif(enb_d = '1') then
			reg_05_3 <= Relu_05;
		elsif(enb_e = '1') then
			reg_05_4 <= Relu_05;
		elsif(enb_f = '1') then
			reg_05_5 <= Relu_05;
		elsif(enb_g = '1') then
			reg_05_6 <= Relu_05;
		elsif(enb_h = '1') then
			reg_05_7 <= Relu_05;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_06_0 <= Relu_06;
		elsif(enb_b = '1') then
			reg_06_1 <= Relu_06;
		elsif(enb_c = '1') then
			reg_06_2 <= Relu_06;
		elsif(enb_d = '1') then
			reg_06_3 <= Relu_06;
		elsif(enb_e = '1') then
			reg_06_4 <= Relu_06;
		elsif(enb_f = '1') then
			reg_06_5 <= Relu_06;
		elsif(enb_g = '1') then
			reg_06_6 <= Relu_06;
		elsif(enb_h = '1') then
			reg_06_7 <= Relu_06;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_07_0 <= Relu_07;
		elsif(enb_b = '1') then
			reg_07_1 <= Relu_07;
		elsif(enb_c = '1') then
			reg_07_2 <= Relu_07;
		elsif(enb_d = '1') then
			reg_07_3 <= Relu_07;
		elsif(enb_e = '1') then
			reg_07_4 <= Relu_07;
		elsif(enb_f = '1') then
			reg_07_5 <= Relu_07;
		elsif(enb_g = '1') then
			reg_07_6 <= Relu_07;
		elsif(enb_h = '1') then
			reg_07_7 <= Relu_07;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_08_0 <= Relu_08;
		elsif(enb_b = '1') then
			reg_08_1 <= Relu_08;
		elsif(enb_c = '1') then
			reg_08_2 <= Relu_08;
		elsif(enb_d = '1') then
			reg_08_3 <= Relu_08;
		elsif(enb_e = '1') then
			reg_08_4 <= Relu_08;
		elsif(enb_f = '1') then
			reg_08_5 <= Relu_08;
		elsif(enb_g = '1') then
			reg_08_6 <= Relu_08;
		elsif(enb_h = '1') then
			reg_08_7 <= Relu_08;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_09_0 <= Relu_09;
		elsif(enb_b = '1') then
			reg_09_1 <= Relu_09;
		elsif(enb_c = '1') then
			reg_09_2 <= Relu_09;
		elsif(enb_d = '1') then
			reg_09_3 <= Relu_09;
		elsif(enb_e = '1') then
			reg_09_4 <= Relu_09;
		elsif(enb_f = '1') then
			reg_09_5 <= Relu_09;
		elsif(enb_g = '1') then
			reg_09_6 <= Relu_09;
		elsif(enb_h = '1') then
			reg_09_7 <= Relu_09;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_10_0 <= Relu_10;
		elsif(enb_b = '1') then
			reg_10_1 <= Relu_10;
		elsif(enb_c = '1') then
			reg_10_2 <= Relu_10;
		elsif(enb_d = '1') then
			reg_10_3 <= Relu_10;
		elsif(enb_e = '1') then
			reg_10_4 <= Relu_10;
		elsif(enb_f = '1') then
			reg_10_5 <= Relu_10;
		elsif(enb_g = '1') then
			reg_10_6 <= Relu_10;
		elsif(enb_h = '1') then
			reg_10_7 <= Relu_10;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_11_0 <= Relu_11;
		elsif(enb_b = '1') then
			reg_11_1 <= Relu_11;
		elsif(enb_c = '1') then
			reg_11_2 <= Relu_11;
		elsif(enb_d = '1') then
			reg_11_3 <= Relu_11;
		elsif(enb_e = '1') then
			reg_11_4 <= Relu_11;
		elsif(enb_f = '1') then
			reg_11_5 <= Relu_11;
		elsif(enb_g = '1') then
			reg_11_6 <= Relu_11;
		elsif(enb_h = '1') then
			reg_11_7 <= Relu_11;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_12_0 <= Relu_12;
		elsif(enb_b = '1') then
			reg_12_1 <= Relu_12;
		elsif(enb_c = '1') then
			reg_12_2 <= Relu_12;
		elsif(enb_d = '1') then
			reg_12_3 <= Relu_12;
		elsif(enb_e = '1') then
			reg_12_4 <= Relu_12;
		elsif(enb_f = '1') then
			reg_12_5 <= Relu_12;
		elsif(enb_g = '1') then
			reg_12_6 <= Relu_12;
		elsif(enb_h = '1') then
			reg_12_7 <= Relu_12;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_13_0 <= Relu_13;
		elsif(enb_b = '1') then
			reg_13_1 <= Relu_13;
		elsif(enb_c = '1') then
			reg_13_2 <= Relu_13;
		elsif(enb_d = '1') then
			reg_13_3 <= Relu_13;
		elsif(enb_e = '1') then
			reg_13_4 <= Relu_13;
		elsif(enb_f = '1') then
			reg_13_5 <= Relu_13;
		elsif(enb_g = '1') then
			reg_13_6 <= Relu_13;
		elsif(enb_h = '1') then
			reg_13_7 <= Relu_13;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_14_0 <= Relu_14;
		elsif(enb_b = '1') then
			reg_14_1 <= Relu_14;
		elsif(enb_c = '1') then
			reg_14_2 <= Relu_14;
		elsif(enb_d = '1') then
			reg_14_3 <= Relu_14;
		elsif(enb_e = '1') then
			reg_14_4 <= Relu_14;
		elsif(enb_f = '1') then
			reg_14_5 <= Relu_14;
		elsif(enb_g = '1') then
			reg_14_6 <= Relu_14;
		elsif(enb_h = '1') then
			reg_14_7 <= Relu_14;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_15_0 <= Relu_15;
		elsif(enb_b = '1') then
			reg_15_1 <= Relu_15;
		elsif(enb_c = '1') then
			reg_15_2 <= Relu_15;
		elsif(enb_d = '1') then
			reg_15_3 <= Relu_15;
		elsif(enb_e = '1') then
			reg_15_4 <= Relu_15;
		elsif(enb_f = '1') then
			reg_15_5 <= Relu_15;
		elsif(enb_g = '1') then
			reg_15_6 <= Relu_15;
		elsif(enb_h = '1') then
			reg_15_7 <= Relu_15;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_16_0 <= Relu_16;
		elsif(enb_b = '1') then
			reg_16_1 <= Relu_16;
		elsif(enb_c = '1') then
			reg_16_2 <= Relu_16;
		elsif(enb_d = '1') then
			reg_16_3 <= Relu_16;
		elsif(enb_e = '1') then
			reg_16_4 <= Relu_16;
		elsif(enb_f = '1') then
			reg_16_5 <= Relu_16;
		elsif(enb_g = '1') then
			reg_16_6 <= Relu_16;
		elsif(enb_h = '1') then
			reg_16_7 <= Relu_16;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_17_0 <= Relu_17;
		elsif(enb_b = '1') then
			reg_17_1 <= Relu_17;
		elsif(enb_c = '1') then
			reg_17_2 <= Relu_17;
		elsif(enb_d = '1') then
			reg_17_3 <= Relu_17;
		elsif(enb_e = '1') then
			reg_17_4 <= Relu_17;
		elsif(enb_f = '1') then
			reg_17_5 <= Relu_17;
		elsif(enb_g = '1') then
			reg_17_6 <= Relu_17;
		elsif(enb_h = '1') then
			reg_17_7 <= Relu_17;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_18_0 <= Relu_18;
		elsif(enb_b = '1') then
			reg_18_1 <= Relu_18;
		elsif(enb_c = '1') then
			reg_18_2 <= Relu_18;
		elsif(enb_d = '1') then
			reg_18_3 <= Relu_18;
		elsif(enb_e = '1') then
			reg_18_4 <= Relu_18;
		elsif(enb_f = '1') then
			reg_18_5 <= Relu_18;
		elsif(enb_g = '1') then
			reg_18_6 <= Relu_18;
		elsif(enb_h = '1') then
			reg_18_7 <= Relu_18;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_19_0 <= Relu_19;
		elsif(enb_b = '1') then
			reg_19_1 <= Relu_19;
		elsif(enb_c = '1') then
			reg_19_2 <= Relu_19;
		elsif(enb_d = '1') then
			reg_19_3 <= Relu_19;
		elsif(enb_e = '1') then
			reg_19_4 <= Relu_19;
		elsif(enb_f = '1') then
			reg_19_5 <= Relu_19;
		elsif(enb_g = '1') then
			reg_19_6 <= Relu_19;
		elsif(enb_h = '1') then
			reg_19_7 <= Relu_19;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_20_0 <= Relu_20;
		elsif(enb_b = '1') then
			reg_20_1 <= Relu_20;
		elsif(enb_c = '1') then
			reg_20_2 <= Relu_20;
		elsif(enb_d = '1') then
			reg_20_3 <= Relu_20;
		elsif(enb_e = '1') then
			reg_20_4 <= Relu_20;
		elsif(enb_f = '1') then
			reg_20_5 <= Relu_20;
		elsif(enb_g = '1') then
			reg_20_6 <= Relu_20;
		elsif(enb_h = '1') then
			reg_20_7 <= Relu_20;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_21_0 <= Relu_21;
		elsif(enb_b = '1') then
			reg_21_1 <= Relu_21;
		elsif(enb_c = '1') then
			reg_21_2 <= Relu_21;
		elsif(enb_d = '1') then
			reg_21_3 <= Relu_21;
		elsif(enb_e = '1') then
			reg_21_4 <= Relu_21;
		elsif(enb_f = '1') then
			reg_21_5 <= Relu_21;
		elsif(enb_g = '1') then
			reg_21_6 <= Relu_21;
		elsif(enb_h = '1') then
			reg_21_7 <= Relu_21;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_22_0 <= Relu_22;
		elsif(enb_b = '1') then
			reg_22_1 <= Relu_22;
		elsif(enb_c = '1') then
			reg_22_2 <= Relu_22;
		elsif(enb_d = '1') then
			reg_22_3 <= Relu_22;
		elsif(enb_e = '1') then
			reg_22_4 <= Relu_22;
		elsif(enb_f = '1') then
			reg_22_5 <= Relu_22;
		elsif(enb_g = '1') then
			reg_22_6 <= Relu_22;
		elsif(enb_h = '1') then
			reg_22_7 <= Relu_22;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_23_0 <= Relu_23;
		elsif(enb_b = '1') then
			reg_23_1 <= Relu_23;
		elsif(enb_c = '1') then
			reg_23_2 <= Relu_23;
		elsif(enb_d = '1') then
			reg_23_3 <= Relu_23;
		elsif(enb_e = '1') then
			reg_23_4 <= Relu_23;
		elsif(enb_f = '1') then
			reg_23_5 <= Relu_23;
		elsif(enb_g = '1') then
			reg_23_6 <= Relu_23;
		elsif(enb_h = '1') then
			reg_23_7 <= Relu_23;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_24_0 <= Relu_24;
		elsif(enb_b = '1') then
			reg_24_1 <= Relu_24;
		elsif(enb_c = '1') then
			reg_24_2 <= Relu_24;
		elsif(enb_d = '1') then
			reg_24_3 <= Relu_24;
		elsif(enb_e = '1') then
			reg_24_4 <= Relu_24;
		elsif(enb_f = '1') then
			reg_24_5 <= Relu_24;
		elsif(enb_g = '1') then
			reg_24_6 <= Relu_24;
		elsif(enb_h = '1') then
			reg_24_7 <= Relu_24;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_25_0 <= Relu_25;
		elsif(enb_b = '1') then
			reg_25_1 <= Relu_25;
		elsif(enb_c = '1') then
			reg_25_2 <= Relu_25;
		elsif(enb_d = '1') then
			reg_25_3 <= Relu_25;
		elsif(enb_e = '1') then
			reg_25_4 <= Relu_25;
		elsif(enb_f = '1') then
			reg_25_5 <= Relu_25;
		elsif(enb_g = '1') then
			reg_25_6 <= Relu_25;
		elsif(enb_h = '1') then
			reg_25_7 <= Relu_25;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_26_0 <= Relu_26;
		elsif(enb_b = '1') then
			reg_26_1 <= Relu_26;
		elsif(enb_c = '1') then
			reg_26_2 <= Relu_26;
		elsif(enb_d = '1') then
			reg_26_3 <= Relu_26;
		elsif(enb_e = '1') then
			reg_26_4 <= Relu_26;
		elsif(enb_f = '1') then
			reg_26_5 <= Relu_26;
		elsif(enb_g = '1') then
			reg_26_6 <= Relu_26;
		elsif(enb_h = '1') then
			reg_26_7 <= Relu_26;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_27_0 <= Relu_27;
		elsif(enb_b = '1') then
			reg_27_1 <= Relu_27;
		elsif(enb_c = '1') then
			reg_27_2 <= Relu_27;
		elsif(enb_d = '1') then
			reg_27_3 <= Relu_27;
		elsif(enb_e = '1') then
			reg_27_4 <= Relu_27;
		elsif(enb_f = '1') then
			reg_27_5 <= Relu_27;
		elsif(enb_g = '1') then
			reg_27_6 <= Relu_27;
		elsif(enb_h = '1') then
			reg_27_7 <= Relu_27;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_28_0 <= Relu_28;
		elsif(enb_b = '1') then
			reg_28_1 <= Relu_28;
		elsif(enb_c = '1') then
			reg_28_2 <= Relu_28;
		elsif(enb_d = '1') then
			reg_28_3 <= Relu_28;
		elsif(enb_e = '1') then
			reg_28_4 <= Relu_28;
		elsif(enb_f = '1') then
			reg_28_5 <= Relu_28;
		elsif(enb_g = '1') then
			reg_28_6 <= Relu_28;
		elsif(enb_h = '1') then
			reg_28_7 <= Relu_28;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_29_0 <= Relu_29;
		elsif(enb_b = '1') then
			reg_29_1 <= Relu_29;
		elsif(enb_c = '1') then
			reg_29_2 <= Relu_29;
		elsif(enb_d = '1') then
			reg_29_3 <= Relu_29;
		elsif(enb_e = '1') then
			reg_29_4 <= Relu_29;
		elsif(enb_f = '1') then
			reg_29_5 <= Relu_29;
		elsif(enb_g = '1') then
			reg_29_6 <= Relu_29;
		elsif(enb_h = '1') then
			reg_29_7 <= Relu_29;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_30_0 <= Relu_30;
		elsif(enb_b = '1') then
			reg_30_1 <= Relu_30;
		elsif(enb_c = '1') then
			reg_30_2 <= Relu_30;
		elsif(enb_d = '1') then
			reg_30_3 <= Relu_30;
		elsif(enb_e = '1') then
			reg_30_4 <= Relu_30;
		elsif(enb_f = '1') then
			reg_30_5 <= Relu_30;
		elsif(enb_g = '1') then
			reg_30_6 <= Relu_30;
		elsif(enb_h = '1') then
			reg_30_7 <= Relu_30;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_31_0 <= Relu_31;
		elsif(enb_b = '1') then
			reg_31_1 <= Relu_31;
		elsif(enb_c = '1') then
			reg_31_2 <= Relu_31;
		elsif(enb_d = '1') then
			reg_31_3 <= Relu_31;
		elsif(enb_e = '1') then
			reg_31_4 <= Relu_31;
		elsif(enb_f = '1') then
			reg_31_5 <= Relu_31;
		elsif(enb_g = '1') then
			reg_31_6 <= Relu_31;
		elsif(enb_h = '1') then
			reg_31_7 <= Relu_31;
		end if;
	end if;  
end process;


---------------------------------------------------------
ROM_W5_inst : ROM_W5 port map (
  CLK => CLK,
  ADDR => W5Addr,
  DATA => W5Data );

---------------------------------------------------------
coeff_00  <= W5Data( 767 downto 765 ) when W5Sel = "000" else W5Data( 764 downto 762 ) when W5Sel = "001" else W5Data( 761 downto 759 ) when W5Sel = "010" else W5Data( 758 downto 756 ) when W5Sel = "011" else W5Data( 755 downto 753 ) when W5Sel = "100" else W5Data( 752 downto 750 ) when W5Sel = "101" else W5Data( 749 downto 747 ) when W5Sel = "110" else W5Data( 746 downto 744 );

coeff_01  <= W5Data( 743 downto 741 ) when W5Sel = "000" else W5Data( 740 downto 738 ) when W5Sel = "001" else W5Data( 737 downto 735 ) when W5Sel = "010" else W5Data( 734 downto 732 ) when W5Sel = "011" else W5Data( 731 downto 729 ) when W5Sel = "100" else W5Data( 728 downto 726 ) when W5Sel = "101" else W5Data( 725 downto 723 ) when W5Sel = "110" else W5Data( 722 downto 720 );

coeff_02  <= W5Data( 719 downto 717 ) when W5Sel = "000" else W5Data( 716 downto 714 ) when W5Sel = "001" else W5Data( 713 downto 711 ) when W5Sel = "010" else W5Data( 710 downto 708 ) when W5Sel = "011" else W5Data( 707 downto 705 ) when W5Sel = "100" else W5Data( 704 downto 702 ) when W5Sel = "101" else W5Data( 701 downto 699 ) when W5Sel = "110" else W5Data( 698 downto 696 );

coeff_03  <= W5Data( 695 downto 693 ) when W5Sel = "000" else W5Data( 692 downto 690 ) when W5Sel = "001" else W5Data( 689 downto 687 ) when W5Sel = "010" else W5Data( 686 downto 684 ) when W5Sel = "011" else W5Data( 683 downto 681 ) when W5Sel = "100" else W5Data( 680 downto 678 ) when W5Sel = "101" else W5Data( 677 downto 675 ) when W5Sel = "110" else W5Data( 674 downto 672 );

coeff_04  <= W5Data( 671 downto 669 ) when W5Sel = "000" else W5Data( 668 downto 666 ) when W5Sel = "001" else W5Data( 665 downto 663 ) when W5Sel = "010" else W5Data( 662 downto 660 ) when W5Sel = "011" else W5Data( 659 downto 657 ) when W5Sel = "100" else W5Data( 656 downto 654 ) when W5Sel = "101" else W5Data( 653 downto 651 ) when W5Sel = "110" else W5Data( 650 downto 648 );

coeff_05  <= W5Data( 647 downto 645 ) when W5Sel = "000" else W5Data( 644 downto 642 ) when W5Sel = "001" else W5Data( 641 downto 639 ) when W5Sel = "010" else W5Data( 638 downto 636 ) when W5Sel = "011" else W5Data( 635 downto 633 ) when W5Sel = "100" else W5Data( 632 downto 630 ) when W5Sel = "101" else W5Data( 629 downto 627 ) when W5Sel = "110" else W5Data( 626 downto 624 );

coeff_06  <= W5Data( 623 downto 621 ) when W5Sel = "000" else W5Data( 620 downto 618 ) when W5Sel = "001" else W5Data( 617 downto 615 ) when W5Sel = "010" else W5Data( 614 downto 612 ) when W5Sel = "011" else W5Data( 611 downto 609 ) when W5Sel = "100" else W5Data( 608 downto 606 ) when W5Sel = "101" else W5Data( 605 downto 603 ) when W5Sel = "110" else W5Data( 602 downto 600 );

coeff_07  <= W5Data( 599 downto 597 ) when W5Sel = "000" else W5Data( 596 downto 594 ) when W5Sel = "001" else W5Data( 593 downto 591 ) when W5Sel = "010" else W5Data( 590 downto 588 ) when W5Sel = "011" else W5Data( 587 downto 585 ) when W5Sel = "100" else W5Data( 584 downto 582 ) when W5Sel = "101" else W5Data( 581 downto 579 ) when W5Sel = "110" else W5Data( 578 downto 576 );

coeff_08  <= W5Data( 575 downto 573 ) when W5Sel = "000" else W5Data( 572 downto 570 ) when W5Sel = "001" else W5Data( 569 downto 567 ) when W5Sel = "010" else W5Data( 566 downto 564 ) when W5Sel = "011" else W5Data( 563 downto 561 ) when W5Sel = "100" else W5Data( 560 downto 558 ) when W5Sel = "101" else W5Data( 557 downto 555 ) when W5Sel = "110" else W5Data( 554 downto 552 );

coeff_09  <= W5Data( 551 downto 549 ) when W5Sel = "000" else W5Data( 548 downto 546 ) when W5Sel = "001" else W5Data( 545 downto 543 ) when W5Sel = "010" else W5Data( 542 downto 540 ) when W5Sel = "011" else W5Data( 539 downto 537 ) when W5Sel = "100" else W5Data( 536 downto 534 ) when W5Sel = "101" else W5Data( 533 downto 531 ) when W5Sel = "110" else W5Data( 530 downto 528 );

coeff_10  <= W5Data( 527 downto 525 ) when W5Sel = "000" else W5Data( 524 downto 522 ) when W5Sel = "001" else W5Data( 521 downto 519 ) when W5Sel = "010" else W5Data( 518 downto 516 ) when W5Sel = "011" else W5Data( 515 downto 513 ) when W5Sel = "100" else W5Data( 512 downto 510 ) when W5Sel = "101" else W5Data( 509 downto 507 ) when W5Sel = "110" else W5Data( 506 downto 504 );

coeff_11  <= W5Data( 503 downto 501 ) when W5Sel = "000" else W5Data( 500 downto 498 ) when W5Sel = "001" else W5Data( 497 downto 495 ) when W5Sel = "010" else W5Data( 494 downto 492 ) when W5Sel = "011" else W5Data( 491 downto 489 ) when W5Sel = "100" else W5Data( 488 downto 486 ) when W5Sel = "101" else W5Data( 485 downto 483 ) when W5Sel = "110" else W5Data( 482 downto 480 );

coeff_12  <= W5Data( 479 downto 477 ) when W5Sel = "000" else W5Data( 476 downto 474 ) when W5Sel = "001" else W5Data( 473 downto 471 ) when W5Sel = "010" else W5Data( 470 downto 468 ) when W5Sel = "011" else W5Data( 467 downto 465 ) when W5Sel = "100" else W5Data( 464 downto 462 ) when W5Sel = "101" else W5Data( 461 downto 459 ) when W5Sel = "110" else W5Data( 458 downto 456 );

coeff_13  <= W5Data( 455 downto 453 ) when W5Sel = "000" else W5Data( 452 downto 450 ) when W5Sel = "001" else W5Data( 449 downto 447 ) when W5Sel = "010" else W5Data( 446 downto 444 ) when W5Sel = "011" else W5Data( 443 downto 441 ) when W5Sel = "100" else W5Data( 440 downto 438 ) when W5Sel = "101" else W5Data( 437 downto 435 ) when W5Sel = "110" else W5Data( 434 downto 432 );

coeff_14  <= W5Data( 431 downto 429 ) when W5Sel = "000" else W5Data( 428 downto 426 ) when W5Sel = "001" else W5Data( 425 downto 423 ) when W5Sel = "010" else W5Data( 422 downto 420 ) when W5Sel = "011" else W5Data( 419 downto 417 ) when W5Sel = "100" else W5Data( 416 downto 414 ) when W5Sel = "101" else W5Data( 413 downto 411 ) when W5Sel = "110" else W5Data( 410 downto 408 );

coeff_15  <= W5Data( 407 downto 405 ) when W5Sel = "000" else W5Data( 404 downto 402 ) when W5Sel = "001" else W5Data( 401 downto 399 ) when W5Sel = "010" else W5Data( 398 downto 396 ) when W5Sel = "011" else W5Data( 395 downto 393 ) when W5Sel = "100" else W5Data( 392 downto 390 ) when W5Sel = "101" else W5Data( 389 downto 387 ) when W5Sel = "110" else W5Data( 386 downto 384 );

coeff_16  <= W5Data( 383 downto 381 ) when W5Sel = "000" else W5Data( 380 downto 378 ) when W5Sel = "001" else W5Data( 377 downto 375 ) when W5Sel = "010" else W5Data( 374 downto 372 ) when W5Sel = "011" else W5Data( 371 downto 369 ) when W5Sel = "100" else W5Data( 368 downto 366 ) when W5Sel = "101" else W5Data( 365 downto 363 ) when W5Sel = "110" else W5Data( 362 downto 360 );

coeff_17  <= W5Data( 359 downto 357 ) when W5Sel = "000" else W5Data( 356 downto 354 ) when W5Sel = "001" else W5Data( 353 downto 351 ) when W5Sel = "010" else W5Data( 350 downto 348 ) when W5Sel = "011" else W5Data( 347 downto 345 ) when W5Sel = "100" else W5Data( 344 downto 342 ) when W5Sel = "101" else W5Data( 341 downto 339 ) when W5Sel = "110" else W5Data( 338 downto 336 );

coeff_18  <= W5Data( 335 downto 333 ) when W5Sel = "000" else W5Data( 332 downto 330 ) when W5Sel = "001" else W5Data( 329 downto 327 ) when W5Sel = "010" else W5Data( 326 downto 324 ) when W5Sel = "011" else W5Data( 323 downto 321 ) when W5Sel = "100" else W5Data( 320 downto 318 ) when W5Sel = "101" else W5Data( 317 downto 315 ) when W5Sel = "110" else W5Data( 314 downto 312 );

coeff_19  <= W5Data( 311 downto 309 ) when W5Sel = "000" else W5Data( 308 downto 306 ) when W5Sel = "001" else W5Data( 305 downto 303 ) when W5Sel = "010" else W5Data( 302 downto 300 ) when W5Sel = "011" else W5Data( 299 downto 297 ) when W5Sel = "100" else W5Data( 296 downto 294 ) when W5Sel = "101" else W5Data( 293 downto 291 ) when W5Sel = "110" else W5Data( 290 downto 288 );

coeff_20  <= W5Data( 287 downto 285 ) when W5Sel = "000" else W5Data( 284 downto 282 ) when W5Sel = "001" else W5Data( 281 downto 279 ) when W5Sel = "010" else W5Data( 278 downto 276 ) when W5Sel = "011" else W5Data( 275 downto 273 ) when W5Sel = "100" else W5Data( 272 downto 270 ) when W5Sel = "101" else W5Data( 269 downto 267 ) when W5Sel = "110" else W5Data( 266 downto 264 );

coeff_21  <= W5Data( 263 downto 261 ) when W5Sel = "000" else W5Data( 260 downto 258 ) when W5Sel = "001" else W5Data( 257 downto 255 ) when W5Sel = "010" else W5Data( 254 downto 252 ) when W5Sel = "011" else W5Data( 251 downto 249 ) when W5Sel = "100" else W5Data( 248 downto 246 ) when W5Sel = "101" else W5Data( 245 downto 243 ) when W5Sel = "110" else W5Data( 242 downto 240 );

coeff_22  <= W5Data( 239 downto 237 ) when W5Sel = "000" else W5Data( 236 downto 234 ) when W5Sel = "001" else W5Data( 233 downto 231 ) when W5Sel = "010" else W5Data( 230 downto 228 ) when W5Sel = "011" else W5Data( 227 downto 225 ) when W5Sel = "100" else W5Data( 224 downto 222 ) when W5Sel = "101" else W5Data( 221 downto 219 ) when W5Sel = "110" else W5Data( 218 downto 216 );

coeff_23  <= W5Data( 215 downto 213 ) when W5Sel = "000" else W5Data( 212 downto 210 ) when W5Sel = "001" else W5Data( 209 downto 207 ) when W5Sel = "010" else W5Data( 206 downto 204 ) when W5Sel = "011" else W5Data( 203 downto 201 ) when W5Sel = "100" else W5Data( 200 downto 198 ) when W5Sel = "101" else W5Data( 197 downto 195 ) when W5Sel = "110" else W5Data( 194 downto 192 );

coeff_24  <= W5Data( 191 downto 189 ) when W5Sel = "000" else W5Data( 188 downto 186 ) when W5Sel = "001" else W5Data( 185 downto 183 ) when W5Sel = "010" else W5Data( 182 downto 180 ) when W5Sel = "011" else W5Data( 179 downto 177 ) when W5Sel = "100" else W5Data( 176 downto 174 ) when W5Sel = "101" else W5Data( 173 downto 171 ) when W5Sel = "110" else W5Data( 170 downto 168 );

coeff_25  <= W5Data( 167 downto 165 ) when W5Sel = "000" else W5Data( 164 downto 162 ) when W5Sel = "001" else W5Data( 161 downto 159 ) when W5Sel = "010" else W5Data( 158 downto 156 ) when W5Sel = "011" else W5Data( 155 downto 153 ) when W5Sel = "100" else W5Data( 152 downto 150 ) when W5Sel = "101" else W5Data( 149 downto 147 ) when W5Sel = "110" else W5Data( 146 downto 144 );

coeff_26  <= W5Data( 143 downto 141 ) when W5Sel = "000" else W5Data( 140 downto 138 ) when W5Sel = "001" else W5Data( 137 downto 135 ) when W5Sel = "010" else W5Data( 134 downto 132 ) when W5Sel = "011" else W5Data( 131 downto 129 ) when W5Sel = "100" else W5Data( 128 downto 126 ) when W5Sel = "101" else W5Data( 125 downto 123 ) when W5Sel = "110" else W5Data( 122 downto 120 );

coeff_27  <= W5Data( 119 downto 117 ) when W5Sel = "000" else W5Data( 116 downto 114 ) when W5Sel = "001" else W5Data( 113 downto 111 ) when W5Sel = "010" else W5Data( 110 downto 108 ) when W5Sel = "011" else W5Data( 107 downto 105 ) when W5Sel = "100" else W5Data( 104 downto 102 ) when W5Sel = "101" else W5Data( 101 downto 99 ) when W5Sel = "110" else W5Data( 98 downto 96 );

coeff_28  <= W5Data( 95 downto 93 ) when W5Sel = "000" else W5Data( 92 downto 90 ) when W5Sel = "001" else W5Data( 89 downto 87 ) when W5Sel = "010" else W5Data( 86 downto 84 ) when W5Sel = "011" else W5Data( 83 downto 81 ) when W5Sel = "100" else W5Data( 80 downto 78 ) when W5Sel = "101" else W5Data( 77 downto 75 ) when W5Sel = "110" else W5Data( 74 downto 72 );

coeff_29  <= W5Data( 71 downto 69 ) when W5Sel = "000" else W5Data( 68 downto 66 ) when W5Sel = "001" else W5Data( 65 downto 63 ) when W5Sel = "010" else W5Data( 62 downto 60 ) when W5Sel = "011" else W5Data( 59 downto 57 ) when W5Sel = "100" else W5Data( 56 downto 54 ) when W5Sel = "101" else W5Data( 53 downto 51 ) when W5Sel = "110" else W5Data( 50 downto 48 );

coeff_30  <= W5Data( 47 downto 45 ) when W5Sel = "000" else W5Data( 44 downto 42 ) when W5Sel = "001" else W5Data( 41 downto 39 ) when W5Sel = "010" else W5Data( 38 downto 36 ) when W5Sel = "011" else W5Data( 35 downto 33 ) when W5Sel = "100" else W5Data( 32 downto 30 ) when W5Sel = "101" else W5Data( 29 downto 27 ) when W5Sel = "110" else W5Data( 26 downto 24 );

coeff_31  <= W5Data( 23 downto 21 ) when W5Sel = "000" else W5Data( 20 downto 18 ) when W5Sel = "001" else W5Data( 17 downto 15 ) when W5Sel = "010" else W5Data( 14 downto 12 ) when W5Sel = "011" else W5Data( 11 downto 9 ) when W5Sel = "100" else W5Data( 8 downto 6 ) when W5Sel = "101" else W5Data( 5 downto 3 ) when W5Sel = "110" else W5Data( 2 downto 0 );


---------------------------------------------------------
dotpro_W5_00 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_00,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_00 );

dotpro_W5_01 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_01,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_01 );

dotpro_W5_02 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_02,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_02 );

dotpro_W5_03 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_03,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_03 );

dotpro_W5_04 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_04,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_04 );

dotpro_W5_05 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_05,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_05 );

dotpro_W5_06 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_06,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_06 );

dotpro_W5_07 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_07,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_07 );

dotpro_W5_08 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_08,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_08 );

dotpro_W5_09 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_09,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_09 );

---------------------------------------------------------
dotpro_W5_10 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_10,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_10 );

dotpro_W5_11 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_11,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_11 );

dotpro_W5_12 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_12,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_12 );

dotpro_W5_13 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_13,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_13 );

dotpro_W5_14 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_14,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_14 );

dotpro_W5_15 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_15,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_15 );

dotpro_W5_16 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_16,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_16 );

dotpro_W5_17 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_17,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_17 );

dotpro_W5_18 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_18,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_18 );

dotpro_W5_19 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_19,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_19 );

---------------------------------------------------------
dotpro_W5_20 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_20,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_20 );

dotpro_W5_21 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_21,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_21 );

dotpro_W5_22 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_22,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_22 );

dotpro_W5_23 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_23,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_23 );

dotpro_W5_24 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_24,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_24 );

dotpro_W5_25 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_25,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_25 );

dotpro_W5_26 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_26,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_26 );

dotpro_W5_27 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_27,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_27 );

dotpro_W5_28 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_28,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_28 );

dotpro_W5_29 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_29,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_29 );

---------------------------------------------------------
dotpro_W5_30 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_30,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_30 );

dotpro_W5_31 : dotpro_W5 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_31,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_31 );

---------------------------------------------------------
  truncate_00 <= mac_00(13 downto 7);
  truncate_01 <= mac_01(13 downto 7);
  truncate_02 <= mac_02(13 downto 7);
  truncate_03 <= mac_03(13 downto 7);
  truncate_04 <= mac_04(13 downto 7);
  truncate_05 <= mac_05(13 downto 7);
  truncate_06 <= mac_06(13 downto 7);
  truncate_07 <= mac_07(13 downto 7);
  truncate_08 <= mac_08(13 downto 7);
  truncate_09 <= mac_09(13 downto 7);
  truncate_10 <= mac_10(13 downto 7);
  truncate_11 <= mac_11(13 downto 7);
  truncate_12 <= mac_12(13 downto 7);
  truncate_13 <= mac_13(13 downto 7);
  truncate_14 <= mac_14(13 downto 7);
  truncate_15 <= mac_15(13 downto 7);
  truncate_16 <= mac_16(13 downto 7);
  truncate_17 <= mac_17(13 downto 7);
  truncate_18 <= mac_18(13 downto 7);
  truncate_19 <= mac_19(13 downto 7);
  truncate_20 <= mac_20(13 downto 7);
  truncate_21 <= mac_21(13 downto 7);
  truncate_22 <= mac_22(13 downto 7);
  truncate_23 <= mac_23(13 downto 7);
  truncate_24 <= mac_24(13 downto 7);
  truncate_25 <= mac_25(13 downto 7);
  truncate_26 <= mac_26(13 downto 7);
  truncate_27 <= mac_27(13 downto 7);
  truncate_28 <= mac_28(13 downto 7);
  truncate_29 <= mac_29(13 downto 7);
  truncate_30 <= mac_30(13 downto 7);
  truncate_31 <= mac_31(13 downto 7);

---------------------------------------------------------
  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);

---------------------------------------------------------
  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);

  Relu_16 <= (others => '0') when truncate_16 < 0 else
             (others => '1') when truncate_16 > 31 else
             truncate_16(4 downto 0);

  Relu_17 <= (others => '0') when truncate_17 < 0 else
             (others => '1') when truncate_17 > 31 else
             truncate_17(4 downto 0);

  Relu_18 <= (others => '0') when truncate_18 < 0 else
             (others => '1') when truncate_18 > 31 else
             truncate_18(4 downto 0);

  Relu_19 <= (others => '0') when truncate_19 < 0 else
             (others => '1') when truncate_19 > 31 else
             truncate_19(4 downto 0);

---------------------------------------------------------
  Relu_20 <= (others => '0') when truncate_20 < 0 else
             (others => '1') when truncate_20 > 31 else
             truncate_20(4 downto 0);

  Relu_21 <= (others => '0') when truncate_21 < 0 else
             (others => '1') when truncate_21 > 31 else
             truncate_21(4 downto 0);

  Relu_22 <= (others => '0') when truncate_22 < 0 else
             (others => '1') when truncate_22 > 31 else
             truncate_22(4 downto 0);

  Relu_23 <= (others => '0') when truncate_23 < 0 else
             (others => '1') when truncate_23 > 31 else
             truncate_23(4 downto 0);

  Relu_24 <= (others => '0') when truncate_24 < 0 else
             (others => '1') when truncate_24 > 31 else
             truncate_24(4 downto 0);

  Relu_25 <= (others => '0') when truncate_25 < 0 else
             (others => '1') when truncate_25 > 31 else
             truncate_25(4 downto 0);

  Relu_26 <= (others => '0') when truncate_26 < 0 else
             (others => '1') when truncate_26 > 31 else
             truncate_26(4 downto 0);

  Relu_27 <= (others => '0') when truncate_27 < 0 else
             (others => '1') when truncate_27 > 31 else
             truncate_27(4 downto 0);

  Relu_28 <= (others => '0') when truncate_28 < 0 else
             (others => '1') when truncate_28 > 31 else
             truncate_28(4 downto 0);

  Relu_29 <= (others => '0') when truncate_29 < 0 else
             (others => '1') when truncate_29 > 31 else
             truncate_29(4 downto 0);

---------------------------------------------------------
  Relu_30 <= (others => '0') when truncate_30 < 0 else
             (others => '1') when truncate_30 > 31 else
             truncate_30(4 downto 0);

  Relu_31 <= (others => '0') when truncate_31 < 0 else
             (others => '1') when truncate_31 > 31 else
             truncate_31(4 downto 0);


--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_00_0 & reg_00_1 & reg_00_2 & reg_00_3 & reg_00_4 & reg_00_5 & reg_00_6 & reg_00_7 & 
	reg_01_0 & reg_01_1 & reg_01_2 & reg_01_3 & reg_01_4 & reg_01_5 & reg_01_6 & reg_01_7 & 
	reg_02_0 & reg_02_1 & reg_02_2 & reg_02_3 & reg_02_4 & reg_02_5 & reg_02_6 & reg_02_7 & 
	reg_03_0 & reg_03_1 & reg_03_2 & reg_03_3 & reg_03_4 & reg_03_5 & reg_03_6 & reg_03_7 & 
	reg_04_0 & reg_04_1 & reg_04_2 & reg_04_3 & reg_04_4 & reg_04_5 & reg_04_6 & reg_04_7 & 
	reg_05_0 & reg_05_1 & reg_05_2 & reg_05_3 & reg_05_4 & reg_05_5 & reg_05_6 & reg_05_7 & 
	reg_06_0 & reg_06_1 & reg_06_2 & reg_06_3 & reg_06_4 & reg_06_5 & reg_06_6 & reg_06_7 & 
	reg_07_0 & reg_07_1 & reg_07_2 & reg_07_3 & reg_07_4 & reg_07_5 & reg_07_6 & reg_07_7 & 
	reg_08_0 & reg_08_1 & reg_08_2 & reg_08_3 & reg_08_4 & reg_08_5 & reg_08_6 & reg_08_7 & 
	reg_09_0 & reg_09_1 & reg_09_2 & reg_09_3 & reg_09_4 & reg_09_5 & reg_09_6 & reg_09_7 & 
	reg_10_0 & reg_10_1 & reg_10_2 & reg_10_3 & reg_10_4 & reg_10_5 & reg_10_6 & reg_10_7 & 
	reg_11_0 & reg_11_1 & reg_11_2 & reg_11_3 & reg_11_4 & reg_11_5 & reg_11_6 & reg_11_7 & 
	reg_12_0 & reg_12_1 & reg_12_2 & reg_12_3 & reg_12_4 & reg_12_5 & reg_12_6 & reg_12_7 & 
	reg_13_0 & reg_13_1 & reg_13_2 & reg_13_3 & reg_13_4 & reg_13_5 & reg_13_6 & reg_13_7 & 
	reg_14_0 & reg_14_1 & reg_14_2 & reg_14_3 & reg_14_4 & reg_14_5 & reg_14_6 & reg_14_7 & 
	reg_15_0 & reg_15_1 & reg_15_2 & reg_15_3 & reg_15_4 & reg_15_5 & reg_15_6 & reg_15_7 & 
	reg_16_0 & reg_16_1 & reg_16_2 & reg_16_3 & reg_16_4 & reg_16_5 & reg_16_6 & reg_16_7 & 
	reg_17_0 & reg_17_1 & reg_17_2 & reg_17_3 & reg_17_4 & reg_17_5 & reg_17_6 & reg_17_7 & 
	reg_18_0 & reg_18_1 & reg_18_2 & reg_18_3 & reg_18_4 & reg_18_5 & reg_18_6 & reg_18_7 & 
	reg_19_0 & reg_19_1 & reg_19_2 & reg_19_3 & reg_19_4 & reg_19_5 & reg_19_6 & reg_19_7 & 
	reg_20_0 & reg_20_1 & reg_20_2 & reg_20_3 & reg_20_4 & reg_20_5 & reg_20_6 & reg_20_7 & 
	reg_21_0 & reg_21_1 & reg_21_2 & reg_21_3 & reg_21_4 & reg_21_5 & reg_21_6 & reg_21_7 & 
	reg_22_0 & reg_22_1 & reg_22_2 & reg_22_3 & reg_22_4 & reg_22_5 & reg_22_6 & reg_22_7 & 
	reg_23_0 & reg_23_1 & reg_23_2 & reg_23_3 & reg_23_4 & reg_23_5 & reg_23_6 & reg_23_7 & 
	reg_24_0 & reg_24_1 & reg_24_2 & reg_24_3 & reg_24_4 & reg_24_5 & reg_24_6 & reg_24_7 & 
	reg_25_0 & reg_25_1 & reg_25_2 & reg_25_3 & reg_25_4 & reg_25_5 & reg_25_6 & reg_25_7 & 
	reg_26_0 & reg_26_1 & reg_26_2 & reg_26_3 & reg_26_4 & reg_26_5 & reg_26_6 & reg_26_7 & 
	reg_27_0 & reg_27_1 & reg_27_2 & reg_27_3 & reg_27_4 & reg_27_5 & reg_27_6 & reg_27_7 & 
	reg_28_0 & reg_28_1 & reg_28_2 & reg_28_3 & reg_28_4 & reg_28_5 & reg_28_6 & reg_28_7 & 
	reg_29_0 & reg_29_1 & reg_29_2 & reg_29_3 & reg_29_4 & reg_29_5 & reg_29_6 & reg_29_7 & 
	reg_30_0 & reg_30_1 & reg_30_2 & reg_30_3 & reg_30_4 & reg_30_5 & reg_30_6 & reg_30_7 & 
	reg_31_0 & reg_31_1 & reg_31_2 & reg_31_3 & reg_31_4 & reg_31_5 & reg_31_6 & reg_31_7 ; 

ReluAllWrN <= not enb_a;

end RTL;

